@@ -108,6 +108,86 @@ group.morph.standard <- function(subjects_dir, subjects_list, measure, hemi='bot
 }
 
 
+#' @title Read combined data for a group from a single file.
+#'
+#' @description Read morphometry data for a group from a matrix in a single MGH or MGZ file.
+#'
+#' @param filepath character string, path to a file in MGH or MGZ format
+#'
+#' @param df logical, whether to return a data.frame, like \code{group.morph.standard}. If FALSE, the raw 4d matrix is returned.
+#'
+#' @return dataframe or 4d matrix, the morph data. See parameter 'df' for details.
+#'
+#' @note The file has typically been generated by running \code{mris_preproc} and/or \code{mri_surf2surf} on the command line, or written from R using \code{\link{write.group.morph.standard.sf}}. The file contains no information on the subject identifiers, you need to know the subjects and their order in the file. Same goes for the hemisphere.
+#'
+#' @seealso If you have created the input data file in FreeSurfer based on an FSGD file, you can read the subject identifiers from that FSGD file using \code{\link{read.md.subjects.from.fsgd}}.
+#'
+#' @export
+group.morph.standard.sf <- function(filepath, df=TRUE) {
+    group_morph_data = freesurferformats::read.fs.volume(filepath);
+    ddim = dim(group_morph_data);
+    if(length(ddim) != 4L) {
+        stop(sprintf("Expected 4-dimensional volume, but file has %d dimensions. Not a group data file?\n", length(ddim)));
+    }
+    if(ddim[2] != 1L | ddim[3] != 1L) {
+        stop(sprintf("Expected data dimension (n, 1, 1, m) with n vertices and m subjects, but found (%d, %d, %d, %d). Not a group data file?\n", ddim[1], ddim[2], ddim[3], ddim[4]));
+    }
+    if(df) {
+        return(unname(as.data.frame(drop(group_morph_data))));
+    } else {
+        return(group_morph_data);
+    }
+}
+
+
+#' @title Write combined data for a group to a single MGH file.
+#'
+#' @description Write morphometry data for a group into a single MGH or MGZ file.
+#'
+#' @param filepath character string, path to the target file, should end with '.mgh' or '.mgz'.
+#'
+#' @param data numerical 4d matrix, with first dimension holding 1D data for all vertices of a subject, 2nd and 3rd dimension are of length 1, and 4th dimenions is for each subject.
+#'
+#' @note The file will contain no information on the subject identifiers. The data can be for one or both hemispheres. See \code{\link{group.morph.standard.sf}} to read the data back into R.
+#'
+#' @export
+write.group.morph.standard.sf <- function(filepath, data) {
+    data = group.data.to.array(data);
+    freesurferformats::write.fs.mgh(filepath, data);
+}
+
+
+#' @title Convert group 1D data to array format.
+#'
+#' @description In general, 1D morphometry data for a group can be stored in a dataframe, a named list, or already a 4D array. This function will convert the given format to matrix format.
+#'
+#' @param data 4D array, named list, or data.frame of group data. The data is expected to be a vector (1D) per subject, as suitable for surface based (vertex-wise) measures.
+#'
+#' @return the array form of the group data. No values are changed, this is only a different data type.
+#'
+#' @keywords internal
+group.data.to.array <- function(data) {
+    if(is.list(data)) {
+        data = as.data.frame(data);
+    }
+    if(is.data.frame(data)) {
+        data = data.matrix(data);
+        ddim = dim(data);
+        if(length(ddim) != 2L) {
+            stop("Invalid dataframe format for 1D group morphometry data.");
+        }
+        dim(data) = c(ddim[1], 1, 1, ddim[2]);
+    }
+    if(! is.array(data)) {
+        stop("Parameter 'data' must be a matrix, named list, or data.frame.");
+    }
+    if(length(dim(data)) != 4L) {
+        stop("Data must have 4 dimensions.");
+    }
+    return(data);
+}
+
+
 #' @title Check whether the subjects_list looks good, warn if not.
 #'
 #' @param subjects_list vector of character strings, the subject IDs.
@@ -191,6 +271,45 @@ group.label <- function(subjects_dir, subjects_list, label, hemi, return_one_bas
 }
 
 
+#' @title Retrieve surface mesh data for a group of subjects.
+#'
+#' @inheritParams group.label
+#'
+#' @param hemi string, one of 'lh', 'rh' or 'both'. The hemisphere name. Used to construct the names of the mesh files to be loaded.
+#'
+#' @param surface character string, the surface to load. Something like 'white' or 'pial'.
+#'
+#' @param force_hemilist logical, whether to force the individual values inside the named return value list to be hemilists (even if the 'hemi' parameter is not set to 'both'). If this is FALSE, the inner values will contain the respective (lh or rh) surface only.
+#'
+#' @return named list of surfaces: Each name is a subject identifier from subjects_list, and the values are hemilists of `fs.surface` instances.
+#'
+#' @family mesh data functions
+#'
+#' @examples
+#' \donttest{
+#'    fsbrain::download_optional_data();
+#'    subjects_dir = fsbrain::get_optional_data_filepath("subjects_dir");
+#'    subjects_list = c("subject1", "subject2");
+#'    surfaces = group.surface(subjects_dir, subjects_list, 'white', "both");
+#' }
+#'
+#' @export
+group.surface <- function(subjects_dir, subjects_list, surface, hemi='both', force_hemilist = TRUE) {
+
+    if(!(hemi %in% c("lh", "rh", "both"))) {
+        stop(sprintf("Parameter 'hemi' must be one of 'lh', 'rh' or 'both' but is '%s'.\n", hemi));
+    }
+
+    check.subjectslist(subjects_list, subjects_dir=subjects_dir);
+
+    all_surfaces = list();
+    for(subject_id in subjects_list) {
+        all_surfaces[[subject_id]] = subject.surface(subjects_dir, subject_id, surface, hemi, force_hemilist = force_hemilist);
+    }
+    return(all_surfaces);
+}
+
+
 
 #'@title Load annotations for a group of subjects.
 #'
