@@ -799,3 +799,86 @@ vis.fs.surface <- function(fs_surface, col="white", per_vertex_data=NULL, hemi="
     }
     return(invisible(vis.coloredmeshes(cm_list, ...)));
 }
+
+
+#' @title Highlight vertices given by index on a subject's meshes.
+#'
+#' @inheritParams vis.color.on.subject
+#'
+#' @inheritParams mesh.vertex.neighbors
+#'
+#' @param verts_lh integer vector, the indices of left hemisphere vertices.
+#'
+#' @param verts_rh integer vector, the indices of right hemisphere vertices.
+#'
+#' @param color_verts_lh vector of colors to visualize on the left hemisphere surface. Length must match number of vertices in 'verts_lh', or be a single color.
+#'
+#' @param color_verts_rh vector of colors to visualize on the right hemisphere surface. Length must match number of vertices in 'verts_rh', or be a single color.
+#'
+#' @param color_bg Background color.
+#'
+#' @param k integer, radius to extend neighborhood (for better visibility).
+#'
+#' @return list of coloredmeshes. The coloredmeshes used for the visualization.
+#'
+#' @family visualization functions
+#' @family surface visualization functions
+#'
+#' @export
+highlight.vertices.on.subject <- function(subjects_dir, vis_subject_id, verts_lh=NULL, verts_rh=NULL, surface="white", views=c('t4'), rgloptions=rglo(), rglactions = list(), color_bg="#FEFEFE", color_verts_lh="#FF0000", color_verts_rh="#FF4500", k=3L) {
+
+    coloredmeshes = list();
+    nv = subject.num.verts(subjects_dir, vis_subject_id);
+
+    color_lh = rep(color_bg, nv$lh);
+    if(length(verts_lh) > 0L) {
+        if(length(color_verts_lh) != length(verts_lh)) {
+            if(length(color_verts_lh) == 1L) {
+                color_verts_lh = rep(color_verts_lh, length(verts_lh));
+            } else {
+                stop(sprintf("Parameter 'color_verts_lh' has length %d, must have length 1 or same length as 'verts_lh', which is %d.", length(color_verts_lh), length(verts_lh)));
+            }
+        }
+        color_lh[verts_lh] = color_verts_lh;
+        if(k > 1L) {
+            # Grow neighborhood and color it in the color of the central vertex.
+            for(vertex_seq_idx in seq(verts_lh)) {
+                vertex_mesh_idx = verts_lh[vertex_seq_idx];
+                neighborhood = mesh.vertex.neighbors(subject.surface(subjects_dir, vis_subject_id, surface = surface, hemi = "lh"), source_vertices = vertex_mesh_idx, k = k)$vertices;
+                color_lh[neighborhood] = color_verts_lh[vertex_seq_idx];
+            }
+        }
+    }
+    cmesh_lh = coloredmesh.from.color(subjects_dir, vis_subject_id, color_lh, 'lh', surface=surface);
+    coloredmeshes$lh = cmesh_lh;
+
+
+    color_rh = rep(color_bg, nv$rh);
+    if(length(verts_rh) > 0L) {
+        if(length(color_verts_rh) != length(verts_rh)) {
+            if(length(color_verts_rh) == 1L) {
+                color_verts_rh = rep(color_verts_rh, length(verts_rh));
+            } else {
+                stop(sprintf("Parameter 'color_verts_rh' has length %d, must have length 1 or same length as 'verts_rh', which is %d.", length(color_verts_rh), length(verts_rh)));
+            }
+        }
+        color_rh[verts_rh] = color_verts_rh;
+        if(k > 1L) {
+            # Grow neighborhood and color it in the color of the central vertex.
+            for(vertex_seq_idx in seq(verts_rh)) {
+                vertex_mesh_idx = verts_rh[vertex_seq_idx];
+                neighborhood = mesh.vertex.neighbors(subject.surface(subjects_dir, vis_subject_id, surface = surface, hemi = "rh"), source_vertices = vertex_mesh_idx, k = k)$vertices;
+                color_rh[neighborhood] = color_verts_rh[vertex_seq_idx];
+            }
+        }
+    }
+    cmesh_rh = coloredmesh.from.color(subjects_dir, vis_subject_id, color_rh, 'rh', surface=surface);
+    coloredmeshes$rh = cmesh_rh;
+
+
+    if(hasIn(rglactions, c('no_vis'))) {
+        return(coloredmeshes);
+    }
+
+    return(invisible(brainviews(views, coloredmeshes, rgloptions = rgloptions, rglactions = rglactions)));
+}
